! gravity5b.f90
! Compute slopes at the nodes (not facets), w. centrifugal force.
! Miroslav Broz (miroslav.broz@email.cz), Sep 21st 2020

program gravity5b

! modules
use const_module
use read_elem_module
use read_face_module
use read_node_module
use volume_module
use centre_module
use normal_module
use normalize_module
use bruteforce_module

implicit none

! constants
double precision, parameter :: pi = 4.d0*atan(1.d0)

! variables
character(len=255) :: f_elem, f_face, f_node
double precision :: rho, unit, scale

double precision :: V, capr, capm
double precision :: x, y, z, U, epsil, alpha
double precision :: P, omega, omega2, l2
double precision, dimension(3) :: r, a, l
real :: t1, t2
integer :: i, j, k, iu, iub
integer, dimension(:,:), pointer :: elems, faces
double precision, dimension(:,:), pointer :: nodes
double precision, dimension(:), pointer :: vols, alphas
double precision, dimension(:,:), pointer :: coms, cofs, normals, nons
integer, dimension(:), pointer :: sums

! read input
write(*,*) '# elem file:'
read(*,*) f_elem
write(*,*) '# face file:'
read(*,*) f_face
write(*,*) '# node file:'
read(*,*) f_node
write(*,*) '# density [kg m^-3]:'
read(*,*) rho
write(*,*) '# smoothing length [unit]:'
read(*,*) epsil
write(*,*) '# unit length [m]:'
read(*,*) unit
write(*,*) '# scale []:'
read(*,*) scale
write(*,*) '# period [day]:'
read(*,*) P
write(*,*)

! unit conversion
P = P*day

! write input
write(*,*) '# f_elem = ', trim(f_elem)
write(*,*) '# f_face = ', trim(f_face)
write(*,*) '# f_node = ', trim(f_node)
write(*,*) '# rho = ', rho, ' kg m^-3'
write(*,*) '# epsil = ', epsil, ' u.'
write(*,*) '# unit = ', unit, ' m'
write(*,*) '# scale = ', scale
write(*,*) '# P = ', P, ' s = ', P/day, ' day'

! input files
call read_elem(f_elem, elems)
call read_face(f_face, faces)
call read_node(f_node, nodes)

write(*,*) '# nelems = ', size(elems,1)
write(*,*) '# nfaces = ', size(faces,1)
write(*,*) '# nnodes = ', size(nodes,1)

allocate(vols(size(elems,1)))
allocate(coms(size(elems,1),3))
allocate(cofs(size(faces,1),3))
allocate(normals(size(faces,1),3))
allocate(nons(size(nodes,1),3))
allocate(sums(size(nodes,1)))
allocate(alphas(size(nodes,1)))

! unit conversion
nodes = nodes*unit*scale  ! km -> m
r = r*unit
epsil = epsil*unit

! volume
V = volume(elems, nodes, vols)
capr = (V/(4.d0/3.d0*pi))**(1.d0/3.d0)
capm = V * rho

write(*,*) '# V = ', V, ' m^3'
write(*,*) '# R = ', capr, ' m = ', capr/unit, ' [unit]'
write(*,*) '# 2R = ', 2.d0*capr, ' m = ', 2.d0*capr/unit, ' [unit]'
write(*,*) '# M = ', capm, ' kg'
write(*,*) '# M = ', 4.64d18, ' kg (Deschamps etal. 2011)'
write(*,*)

! centres of mass, centres of faces, normals
call centre(elems, nodes, coms)
call centre(faces, nodes, cofs)
call normal(faces, nodes, normals)

! normals of nodes
sums = 0
nons = 0.d0
do i = 1, size(faces,1)
  do j = 1, size(faces,2)
    k = faces(i,j)
    nons(k,:) = nons(k,:) + normals(i,:)
    sums(k) = sums(k)+1
  enddo
enddo

do i = 1, size(nodes,1)
  if (sums(i).gt.0) then
    nons(i,:) = nons(i,:)/sums(i)  ! not all nodes are surface nodes
  endif
enddo

! potential
write(*,*) '# x y z nx ny nz U ax ay az alpha'
write(*,*) '# [m] [m] [m] [] [] [] [J kg^-1] [m s^-2] [..] [..] [rad]'

omega = 2.d0*pi/P
omega2 = omega*omega
write(*,*) '# omega = ', omega, ' rad s^-1'

call cpu_time(t1)
do i = 1, size(nodes,1)
  if (sums(i).gt.0) then
    r = nodes(i,:)
    l = (/r(1), r(2), 0.d0/)
    l2 = dot_product(l, l)
    U = bruteforce(vols, coms, rho, epsil, r) + 1.d0/2.d0*omega2*l2
    a = a_g_bf(vols, coms, rho, r) + omega2*l
    alpha = acos(dot_product(nons(i,:), -normalize(a)))
    alphas(i) = alpha

    write(*,*) r, nons(i,:), U, a, alpha
  endif
enddo
call cpu_time(t2)

write(*,*) '# cpu_time = ', t2-t1, ' s'
write(*,*)

iu = 10
open(unit=iu, file='gravity5.tmp', status='unknown')
write(iu,*) '# x y z alpha'
write(iu,*) '# [m] [m] [m] [rad]'
write(iu,*) '# Gnuplot pm3d format'

do i = 1, size(faces,1)
  write(iu,*) nodes(faces(i,1),:), alphas(faces(i,1))
  write(iu,*) nodes(faces(i,2),:), alphas(faces(i,2))
  write(iu,*)
  write(iu,*) nodes(faces(i,3),:), alphas(faces(i,3))
  write(iu,*) nodes(faces(i,3),:), alphas(faces(i,3))
  write(iu,*)
  write(iu,*)
enddo

close(iu)

open(unit=iu, file='gravity5.tmp2', status='unknown')

do i = 1, size(faces,1)
  write(iu,*) nodes(faces(i,1),:), alphas(faces(i,1))
  write(iu,*) nodes(faces(i,2),:), alphas(faces(i,2))
  write(iu,*) nodes(faces(i,3),:), alphas(faces(i,3))
  write(iu,*) nodes(faces(i,1),:), alphas(faces(i,1))
  write(iu,*)
enddo

close(iu)

open(unit=iu, file='gravity5.tmp3', status='unknown')

do i = 1, size(nodes,1)
  if (sums(i).gt.0) then
    write(iu,*) alphas(i)
  endif
enddo

close(iu)

stop
end program gravity5b


